package WebFlow.ContextManager;

import WebFlow.*;
import WebFlow.event.*;
import WebFlow.hashtable.*;
import WebFlow.RemoteFile.*;
import WebFlow.ContextManager.ContextManagerPackage.*;
import org.omg.CORBA.*;
import java.util.*;
import java.io.*; 

/* author: Tomasz Haupt, NPAC at Syracuse University */

public class ContextManagerImp extends WebFlow.BeanContextChildSupport 
            implements ContextManagerOperations {

  String msg_ ; 
  org.omg.CORBA.Object peer ;
  SaveContext saveContext;
  RemoteFile remotefile;
  WebFlowContext uc,currentP,currentS,currentA;
  String ucID;

  public ContextManagerImp(org.omg.CORBA.Object peer, String msg) 
         throws WebFlow.NullPointerException {

  super(peer);
  this.peer = peer;
  String msg_ = msg;
  System.out.println("end of constructor");
 }

  public void test() {
        System.out.println("context manager is up");
  }

  public void init(WebFlowContext uc, org.omg.CORBA.Object sco, 
                                      org.omg.CORBA.Object rfo){
   this.uc = uc;
   saveContext = SaveContextHelper.narrow(sco);
   remotefile = RemoteFileHelper.narrow(rfo);
   currentP = null;
   currentS = null;
   currentA = null;
   ucID = uc.getObjectID();
  }

  public WebFlowContext createGatewayContext(WebFlowContext p, String name){
    try {
    WebFlowContext c = WebFlowContextHelper.narrow(p.addNewContext(name));
    if(c==null) System.out.println("c is null");
    ContextData cd = ContextDataHelper.narrow(c.addNewModule("ContextData"));
    if(cd==null) System.out.println("--->cd is null");
    return c;
    } catch (Exception e) {
       System.out.println(e +"while createGatewayContext");
       return null;
    }  
  }

  public void restoreChildContexts(WebFlowContext parent){
    String[] children = listChildrenContexts(parent);
    if(!(children[0].equals("null"))) {
      for(int i=0;i<children.length;i++) {
        WebFlowContext c = createGatewayContext(parent,children[i]);
        saveContext.restoreContextData(c,"/tmp/"); 
        restoreChildContexts(c);
      }  
    }
  }

  public boolean setCurrentContexts(){
    currentP = null;
    currentS = null;
    currentA = null;
    boolean status = true;
    String ContextName = getContextData(uc,"CurrentChild");
    if(!ContextName.equals("null")) {
      int idx= ContextName.lastIndexOf(".");
      if(idx>-1) { 
         status = setCurrentProblem(ContextName.substring(0,idx)); 
         if(status) {
           ContextName = getCurrentProblemProperty("CurrentChild");
           if(!ContextName.equals("null")) {
              idx= ContextName.lastIndexOf(".");
              if(idx>-1) { 
                  status = setCurrentSession(ContextName.substring(0,idx)); 
                  if(status) {
                     ContextName = getCurrentSessionProperty("CurrentChild");
                     if(!ContextName.equals("null")) {
                       idx= ContextName.lastIndexOf(".");
                         if(idx>-1) { 
                             status = setCurrentApplication(
                                      ContextName.substring(0,idx));
			 }
                         else {
                             System.out.println("cannot find suffix");
                             status = false;
			 }
		     } 
		  }
	      }
              else {
                 System.out.println("cannot find suffix");
                 status = false;
	      }
	   }
	 }
      }
      else {
         System.out.println("cannot find suffix");
         status = false;
      }
    } 
    return status;
  }

  public void setContextData(WebFlowContext c, String name, String value){
     ContextData cd = findContextData(c);
     cd.setParam(name,value);
  }
  public void setContextMData(WebFlowContext c, String[] name, String[] value){
     ContextData cd = findContextData(c);
     for(int i=0;i<name.length;i++) { 
       cd.setParam(name[i],value[i]);
     }
  }

  public String getContextData(WebFlowContext c, String name){
     ContextData cd = findContextData(c);
     String value = cd.getParam(name); 
     return value;
  }

  public String[] listChildren(WebFlowContext c){
     String[] result = null; 
     String Children = getContextData(c,"Children");
     if(Children.equals("null")) {
         result = new String[1];
         result[0] = "null";
     }
     else {
         StringTokenizer st = new StringTokenizer(Children,"+");
         int n = st.countTokens();
         result = new String[n]; 
         for (int i=0; i<n; i++) {
            String name = st.nextToken();
            result[i]=name.substring(0,name.lastIndexOf("."));
	 }
     }
     return result;
  } 

  public String[] listChildrenContexts(WebFlowContext c){
     String[] result = null; 
     String Children = getContextData(c,"Children");
     if(Children.length()<1) Children="null";
     if(Children.equals("null")) {
         result = new String[1];
         result[0] = "null";
     }
     else {
         StringTokenizer st = new StringTokenizer(Children,"+");
         int n = st.countTokens();
         result = new String[n]; 
         for (int i=0; i<n; i++) {
            result[i]=st.nextToken();
	 }
     }
     return result;
  }

  public String[] listModules(WebFlowContext c){
     System.out.println("listModules not yet implemented");
     String[] result = new String[1];
     result[0]="null";
     return result;
  } 

  public WebFlowContext getContext(String FullContextName){
     WebFlowContext c = null;
     org.omg.CORBA.Object o = uc.getContext(FullContextName);
     if(o == null) {
        System.out.println("cannot find context "+FullContextName);
     }
     c = WebFlowContextHelper.narrow(o);
     return c;
  } 

  public String getContextID(String type){
     String result = "none";
     if(type.equals("PC")) result = currentP.getObjectID();
     if(type.equals("SC")) result = currentS.getObjectID();
     if(type.equals("AC")) result = currentA.getObjectID();
     return result;
  }


    /* Gateway specific methods */

  public boolean setProblemContext(String problem){
     boolean status=true;
     WebFlowContext c = null;
     String id = ucID + "/"+problem+".PC";
     c=getContext(id);
     System.out.println("*** returned from getContext");
     if(c == null) {
        status=false;
        System.out.println("failed to get "+id);
     }
     else {
        System.out.println(c.getObjectID());
        currentP = c;
     }           
     return status;
  } 

  public boolean setSessionContext(String Session){
     boolean status=true;
     WebFlowContext c = null;
     String id = currentP.getObjectID() + "/"+Session+".SC";
     c=getContext(id);
     System.out.println("*** returned from getContext");
     if(c == null) {
        status=false;
        System.out.println("failed to get "+id);
     }
     else {
        System.out.println(c.getObjectID());
        currentS = c;
     }           
     return status;
  } 

  public boolean addNewProblem(String ProjContextName){
     boolean status = true;
     status = setNewProblem(uc,ProjContextName);
     return status;
  }

  public boolean addProblem(String ProjContextName){
     boolean status = true;
     status = setNewProblem(uc,ProjContextName);
     return status;
  }

  public boolean setNewProblem(WebFlowContext uc, String ProjContextName){

     boolean status=true;
     ProjContextName=removeSpaces(ProjContextName);
     String ProblemName=setShortName(ProjContextName);
     String curDir = getContextData(uc,"Directory")+"/"+ProblemName;

     if(remotefile.mkdir(curDir)){
         ProjContextName += ".PC";
         setContextData(uc,"CurrentChild",ProjContextName);
         WebFlowContext p = createGatewayContext(uc,ProjContextName);
         Date currentDate = new Date();
         long time = currentDate.getTime(); 
         String timestring = (new Long(time).toString());
         String parentID = uc.getObjectID(); 

         String[] names={"Date", "LastTime", "Directory", "Name",
                       "ContextName", "CurrentChild", "Parent", "Descriptor"};
         String[] values={timestring,timestring,curDir,ProblemName,
                          ProjContextName,"null",parentID,"null"};

         setSessionMData(p,names,values);  
         updateChildren(uc,ProjContextName);
         currentP = p;
     }
      else {
        System.out.println("NewProblem: already exists = " + ProjContextName);
        status=false; 
      }
    return status;
  }


  public boolean removeContext(WebFlowContext c){
    /* update Children filed in the Context data;
       remove jobs; ? separate call?
       remove Problem directory;
       update CurrentChild, if appropriate 
       remove context itself */

    boolean status = true;
    String ContextName = getContextData(c,"ContextName");
    String curDir = getContextData(c,"Directory");
    String parentID = getContextData(c,"Parent");
    WebFlowContext parent = getContext(parentID);
    if(parent==null) {
       System.out.println("cannot get parent context "+parentID);
       status = false;
    }
    else { 
       rmChild(parent, ContextName);
       String currentChild = getContextData(parent,"CurrentChild");
       if(currentChild.equals(ContextName)) {
          setSessionData(parent,"CurrentChild","null");
       }
       if(!remotefile.rmdir(curDir)) {
         System.out.println("deletion of "+curDir+" failed!");
         status=false;
       } else {
         c.removeMyself();
       }
    }               
    return status;
  }


  public boolean removeCurrentProblem(){
       boolean status = removeContext(currentP);
       currentP=null;
       currentS=null;
       currentA=null;
       setSessionData(uc,"CurrentChild","null");
       return status;
  }  

  public boolean removeProblem(String problem){ 
       boolean status = true;
       String id = ucID+"/"+problem+".PC";
       WebFlowContext c = getContext(id);
       if(c==null) {
          System.out.println("cannot find session context "+id);
          status=false;
       }  
       else {
          status = removeContext(c);
          currentP = null;
          currentS = null;
          currentA = null;
          setSessionData(uc,"CurrentChild","null");
       }
       return status; 
  }

  public boolean removeCurrentSession(){
       boolean status = removeContext(currentS);
       currentS=null;
       currentA=null;
       setSessionData(currentP,"CurrentChild","null");
       return status;
  }  

  public boolean removeCurrentApplication(){
       boolean status = removeContext(currentA);
       currentA=null;
       setSessionData(currentS,"CurrentChild","null");
       return status;
  }  

  public boolean removeCurrentProblemChild(String session) {
       boolean status = true;
       String id = currentP.getObjectID()+"/"+session+".SC";
       WebFlowContext c = getContext(id);
       if(c==null) {
          System.out.println("cannot find session context "+id);
          status=false;
       }  
       else {
          status = removeContext(c);
          currentS = null;
          currentA = null;
          setSessionData(currentP,"CurrentChild","null");
       }
       return status; 
  }

  public boolean removeCurrentSessionChild(String application) {
       boolean status = true;
       String id = currentS.getObjectID()+"/"+application+".AC";
       WebFlowContext c = getContext(id);
       if(c==null) {
          System.out.println("cannot find application context "+id);
          status=false;
       }  
       else {
          status = removeContext(c);
          setSessionData(currentS,"CurrentChild","null");
          currentA = null;
       }
       return status; 
  }


  public boolean addNewSession(String problem, String SessionContextName){
        boolean status = true;
        String key = "none";
        WebFlowContext p = null;
        try {
          key = ucID+"/"+problem; 
          org.omg.CORBA.Object o = uc.getKeyedChild(key); 
          p = WebFlowContextHelper.narrow(o);
          if(p != null) {
          status = setNewSession(p,SessionContextName);
	  }
          else {
            System.out.println("Cannot find context: "+key);
            status=false;
	  } 
        } catch (Exception e) {
            System.out.println("EXCEPTION in addNewSession: "+e);
            status = false;
	} 
        return status;
  }

  public boolean addSession(String SessionContextName){
        boolean status = true;
        if(currentP != null) {
          status = setNewSession(currentP,SessionContextName);
	  }
          else {
            System.out.println("no current context");
            status=false;
	  } 
        return status;
  }

  public boolean setNewSession(WebFlowContext pc, String SessionContextName){

     boolean status=true;  
     SessionContextName=removeSpaces(SessionContextName);   
     String SessionName=setShortName(SessionContextName);
     String curDir=getContextData(pc,"Directory") + "/"+SessionName;
     String Problem = getContextData(pc,"Problem"); 
     if(remotefile.mkdir(curDir)) {  
       SessionContextName += ".SC";
       setContextData(pc,"CurrentChild",SessionContextName);
       WebFlowContext s = createGatewayContext(pc,SessionContextName);
       Date currentDate = new Date();
       long time = currentDate.getTime(); 
       String timestring = (new Long(time).toString());
       String parentID = pc.getObjectID(); 

       String[] names={"Date", "LastTime", "Directory", "Name",
                       "ContextName", "CurrentChild", "Parent", "Descriptor"};
       String[] values={timestring,timestring,curDir, SessionName,
                        SessionContextName,"null",parentID,"null"};

       setSessionMData(s,names,values);
       updateChildren(pc,SessionContextName);
       currentS = s;
     }
     else {
           System.out.println("NewSession: already exists");
       status=false;  
     }
  return status;   
  }

  public boolean addNewApplication(String problem, 
                                String session, String ApplContextName){
        boolean status = true;
        String key = "none";
        WebFlowContext s = null;
        try {
          key = ucID+"/"+problem+"/"+session; 
          org.omg.CORBA.Object o = uc.getContext(key); 
          s = WebFlowContextHelper.narrow(o);
          if(s != null) {
            status = setNewApplication(s,ApplContextName);
	  }
          else {
            System.out.println("Cannot find context: "+key);
            status=false;
	  } 
        } catch (Exception e) {
            System.out.println("EXCEPTION in addNewSession: "+e);
            status = false;
	} 
        return status;
  }

  public boolean addApplication(String ApplContextName){
        boolean status = true;
        if(currentS != null) {
            status = setNewApplication(currentS,ApplContextName);
	  }
          else {
            System.out.println("no current context");
            status=false;
	  } 
        return status;
  }

  public boolean setNewApplication(WebFlowContext sc, String ApplContextName){
     boolean status=true;

     String appl = ApplContextName;
     ApplContextName=removeSpaces(ApplContextName);   
     String ApplicationName=setShortName(ApplContextName);
     String curDir=getContextData(sc,"Directory")+ "/"+ApplicationName;
     String Session = getContextData(sc,"Session");
     String problem = getContextData(sc,"Problem");

     if(remotefile.mkdir(curDir)) { 
       ApplContextName += ".AC";
       setContextData(sc,"CurrentChild",ApplContextName);
       WebFlowContext a = createGatewayContext(sc,ApplContextName); 
       Date currentDate = new Date();
       long time = currentDate.getTime(); 
       String timestring = (new Long(time).toString());
       String parentID = sc.getObjectID(); 

       String[] names={"Date", "LastTime", "Directory", "Name",
                       "ContextName", "CurrentChild", "Parent", "Descriptor"};
       String[] values={timestring,timestring,curDir, ApplicationName,
                        ApplContextName,"null",parentID,appl};

       setSessionMData(a,names,values);     
       updateChildren(sc,ApplContextName);
       currentA = a;
     }
     else {
       System.out.println("NewApplication: already exists");
       status = false;
     }      
     return status;
  }

  public void setSessionData(WebFlowContext c, String name, String value){ 
     //set data and last updated time; save context data

     ContextData cd = findContextData(c);
     cd.setParam(name,value);
     long time = (new Date()).getTime();
     String timestring = (new Long(time).toString());
     cd.setParam("LastTime",timestring);

     saveContext.saveContextData(c);
  }    
  public void setSessionMData(WebFlowContext c, String[] name, String[] value){
 
     //set data and last updated time; save context data

     int n = name.length; 
     ContextData cd = findContextData(c);
     for(int i=0;i<n;i++) {  
       cd.setParam(name[i],value[i]);
     }
     long time = (new Date()).getTime();
     String timestring = (new Long(time).toString());
     cd.setParam("LastTime",timestring);
     saveContext.saveContextData(c);
  }    

  public String getCurrentProblemProperty(String name) {
     String value="null";
     if(currentP == null) {
       value = "nocurrentcontext";
     }
     else {
       value = getContextData(currentP, name);
     }
     return value;  
  }

  public String getProblemProperty(String problem, String name){
     String id = ucID + "/"+problem+".PC";
     WebFlowContext c = getContext(id); 
     String value = "null";
     if(c == null) {
        value = "nocontextfound";
     }
     else {
        value = getContextData(c,name);
     }
     return value;
  } 


  public String getCurrentSessionProperty(String name) {
     String value="null";
     if(currentS == null) {
       value = "nocurrentcontext";
     }
     else {  
       value = getContextData(currentS, name);
     }
     return value;  
  }

  public String getSessionProperty(String problem, String session, 
                                   String name){
     String id = ucID + "/"+problem+".PC" +"/"+session+".SC";
     WebFlowContext c = getContext(id); 
     String value = "null";
     if(c == null) {
        value = "nocontextfound";
     }
     else {
        value = getContextData(c,name);
     }
     return value;
  } 

  public String getCurrentApplicationProperty(String name) {
     String value="null";
     if(currentA == null) {
       value = "nocurrentcontext";
     }
     else {
       value = getContextData(currentA, name);
     }
     return value;  
  }

  public String getApplicationProperty(String problem, String session, 
                                   String application, String name){
     String id = ucID + "/"+problem+".PC" +"/"+session+".SC";
     id += "/"+application+".AC";
     WebFlowContext c = getContext(id); 
     String value = "null";
     if(c == null) {
        value = "nocontextfound";
     }
     else {
        value = getContextData(c,name);
     }
     return value;
  } 

  public boolean setCurrentProblemProperty(String name, String value) {
     boolean status = false;
     if(currentP != null) {
       status = true; 
       setSessionData(currentP, name, value);
     }
     return status;  
  }

  public boolean setCurrentSessionProperty(String name, String value) {
     boolean status = false;
     if(currentS != null) {
       status = true; 
       setSessionData(currentS, name, value);
     }
     return status;  
  }

  public boolean setCurrentApplicationProperty(String name, String value) {
     boolean status = false;
     if(currentA != null) {
       status = true; 
       setSessionData(currentA, name, value);
     }
     return status;  
  }

  public boolean setCurrentProblem(String problem){
    boolean status = true;
    String id = ucID + "/"+problem+".PC";
    WebFlowContext c = getContext(id);
    if(c == null) {
      System.out.println("cannot find context "+id);
      status=false;
    }
    else{
       currentP = c; 
       currentS = null;
       currentA = null;
       String contextName = getContextData(currentP,"ContextName");
       setContextData(uc,"CurrentChild",contextName);
    }
    return status;
  }

  public boolean setCurrentSession(String session){
    boolean status = true;
    String id = currentP.getObjectID();
    id += "/"+session+".SC";
    WebFlowContext c = getContext(id);
    if(c == null) {
      System.out.println("cannot find context "+id);
      status=false;
    }
    else{
       currentS = c; 
       currentA = null;
       String contextName = getContextData(currentS,"ContextName");
       setContextData(currentP,"CurrentChild",contextName);
    }
    return status;
  }

  public boolean setCurrentApplication(String application){
    boolean status = true;
    if(currentS==null) {
        System.out.println("...currentS is null");
        status = false;
        return status;
    }
    String id = currentS.getObjectID();
    id += "/"+application+".AC";
    WebFlowContext c = getContext(id);
    if(c == null) {
      System.out.println("cannot find context "+id);
      status=false;
    }
    else{
       currentA = c; 
       String contextName = getContextData(currentA,"ContextName");
       setContextData(currentS,"CurrentChild",contextName);
    }
    return status;
  }

  public boolean selectProblem(String problem) { 
    boolean status = true;
    String id = ucID + "/"+problem+".PC";
    WebFlowContext c = getContext(id);
    if(c == null) {
      System.out.println("cannot find context "+id);
      status=false;
    }
    else{
       currentP = c; 
       setContextData(uc,"CurrentChild",problem+".PC");
       currentS = null;
       currentA = null;
    }
    return status;
  }

  public boolean selectSession(String problem, String session) { 
    boolean status = true;
    String id = ucID + "/"+problem+".PC/"+session+".SC";
    WebFlowContext c = getContext(id);
    if(c == null) {
      System.out.println("cannot find context "+id);
      status=false;
    }
    else{
       currentS = c; 
       id =  ucID + "/"+problem+".PC";
       c = getContext(id);
       currentP = c; 
       setContextData(uc,"CurrentChild",problem+".PC");
       setContextData(c,"CurrentChild", session+".SC");
       currentA = null;
    }
    return status;
  }

  public boolean selectApplication(String problem, String session,
                                   String application) { 
    boolean status = true;
    String id = ucID + "/"+problem+".PC/"+session+".SC/"+application+".AC";
    WebFlowContext c = getContext(id);
    if(c == null) {
      System.out.println("cannot find context "+id);
      status=false;
    }
    else{
       currentA = c; 
       id =  ucID + "/"+problem+".PC";
       c = getContext(id);
       currentP = c;
       id =  ucID + "/"+problem+".PC/"+session+"/.SC";
       c = getContext(id);
       currentS = c;
        
       setContextData(uc,"CurrentChild",problem+".PC");
       setContextData(currentP,"CurrentChild", session+".SC");
       setContextData(currentS,"CurrentChild", application+".AC");
    }
    return status;
  }


  public boolean isCurrentProblem() {
     boolean status = true;
     if(currentP == null) status =false;
     return status;
  }

  public boolean isCurrentSession() {
     boolean status = true;
     if(currentS == null) status =false;
     return status;
  }

  public boolean isCurrentApplication() {
     boolean status = true;
     if(currentA == null) status =false;
     return status;
  }

  public String[] listProblems(){
     String[] result = listChildren(uc);
     return result; 
  }

  public String[] listSessions(){
     String[] result = listChildren(currentP);
     return result; 
  }

  public String[] listApplications(){
     String[] result = listChildren(currentS);
     return result; 
  }

  public String[] listProblemSessions(String problem){
     String[] result = null;
     WebFlowContext c = getContext(ucID+"/"+problem+".PC");
     if(c==null) {
       System.out.println("cannot find context "+ucID+"/"+problem+".PC");
       result = new String[1];
       result[0]="null";
     }
     else {
       result = listChildren(c);
     }
     return result; 
  }

  public String[] listSessionApplications(String session){
     String[] result = null;
     String id = currentP.getObjectID()+"/"+session+".PC";
     WebFlowContext c = getContext(id);
     if(c==null) {
       System.out.println("cannot find context "+id);
       result = new String[1];
       result[0]="null";
     }
     else {
       result = listChildren(c);
     }
     return result; 
  }

  public String[] listAllSessions(){
     Vector v = new Vector();
     String[] result = null;

     String[] problems = listProblems();
     if(problems[0].equals("null")) {
          result = new String[1];
          result[0]="null";
     }
     else {        
        int n = problems.length;
        WebFlowContext pc = null; 
        for(int i=0;i<n;i++) {
           String id = ucID+"/"+problems[i]+".PC";
           pc = getContext(id);
           if(pc==null) {
             System.out.println("cannot find context "+id);
             result = new String[1];
             result[0]="null";
	   }
           else {
             String[] r = listChildren(pc);
             if(!(r[0].equals("null"))) { 
                for(int j=0;j<r.length;j++) {
                  v.addElement(problems[i]+"/"+r[j]);
		}
	     }
	   }
	}
     }
     int nn = v.size();
     if(nn<1) {
       result = new String[1];
       result[0]="null";
     }
     else {
       result = new String[nn]; 
       for(int k=0;k<nn;k++) result[k]=(String)v.get(k);
     } 
     return result; 
  }

  public String[] listAllApplications(){
     Vector v = new Vector();
     String[] result = null;

     String[] problems = listProblems();
     if(problems[0].equals("null")) {
          result = new String[1];
          result[0]="null";
     }
     else {        
        int n = problems.length;
        WebFlowContext pc = null; 
        for(int i=0;i<n;i++) {
           String id = ucID+"/"+problems[i]+".PC";
           pc = getContext(id);
           if(pc==null) {
             System.out.println("cannot find problem context "+id);
             result = new String[1];
             result[0]="null";
	   }
           else {
             String[] sessions = listChildren(pc);
             if(!(sessions[0].equals("null"))) {
                int ns = sessions.length;
                WebFlowContext sc = null; 
                for(int ii=0;ii<ns;ii++) { 
                   String id1 = id + "/"+sessions[ii]+".SC";
                   sc = getContext(id1);
                   if(sc==null) {
                     System.out.println("cannot find session context "+id1);
                   }
                   else {
                     String[] r = listChildren(sc);
                     if(!(r[0].equals("null"))) {
                      int na = r.length;    
                       for(int j=0;j<r.length;j++) {
                         v.addElement(problems[i]+"/"+sessions[ii]+"/"+r[j]);
		       }
		     }
		   }
		}
	     }
	   }
	}
     }
     int nn = v.size();
     if(nn<1) {
       result = new String[1];
       result[0]="null";
     }
     else {
       result = new String[nn]; 
       for(int k=0;k<nn;k++) result[k]=(String)v.get(k);
     } 
     return result; 
  }

    /* Convenience methods */
    
  public void createNewProblem(WebFlowContext root, String name){};
  public void createNewSession(WebFlowContext root, String name){};
  public void createNewApplication(WebFlowContext root, String name){};
  public void cloneSession(WebFlowContext oldSession){};
  public void replaceProblem(){};    

          /* utilities */

  public String removeSpaces(String name){ 
    // if name contains spaces replace them by _
    String oldName = name; 
      while(oldName.indexOf(' ')>-1) {
        StringBuffer sb = new StringBuffer(oldName);
        int idx1=oldName.indexOf(' ');
        sb.setCharAt(idx1,'_');
        oldName=new String(sb); 
    }
    String newName=oldName;
    return newName; 
  } 

  public String setShortName(String ContextName){
          // format ContextName for display
    String ShortName = ContextName;
    int idx=ShortName.lastIndexOf("/");
    ShortName = ShortName.substring(idx+1);
    return ShortName; 
  }


  private void updateChildren(WebFlowContext c, String ContextName){ 
    String Children = getContextData(c,"Children");
    if(Children.equals("")) {
       setSessionData(c,"Children", ContextName);
    }
    else {
       setSessionData(c,"Children", Children+"+"+ContextName);
    }         
  }

  private void rmChild(WebFlowContext c, String contextName) {
    String children = getContextData(c,"Children");
    int idx=children.indexOf(contextName);
    int l=contextName.length();
    String newChildren=""; 
    if(idx>0) newChildren = children.substring(0,idx-1);
      newChildren += children.substring(idx+l);
      if(newChildren.startsWith("+")) newChildren=newChildren.substring(1);
      setSessionData(c, "Children", newChildren); 
  }
 
  private ContextData findContextData(WebFlowContext wfc){
     ContextData wfcContextData=null;

     String ContextDataName = wfc.getObjectID()+"/ContextData";
     org.omg.CORBA.Object o = wfc.getKeyedChild(ContextDataName);
     if(o != null) {
          wfcContextData = ContextDataHelper.narrow(o); 
     }
     else {System.out.println("Context data not found!");}

     
     if(wfcContextData==null) {
        System.out.println("Context data not found!");
     }
        return wfcContextData;
  }



  public static void main(String args[]){
      System.out.println("hello from ContextManager");
  }
}







